<?php
/**
 * Zarafa Z-Merge SugarCRM Connector
 *
 * Copyright (C) 2005 - 2008  Zarafa B.V.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
 * and under the terms of the GNU General Public License, version 3,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU (Affero) General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * and the GNU General Public License along with this program.
 * If not, see <http://www.gnu.org/licenses/>.
 *
 * Created on 07.11.2007 by Sebastian Kummer & Manfred Kutas
 *
 * NOTE: action performs a check on the installation and
 * returns status information
 *
 * @package Z-Merge
 * @subpackage SugarCRM-Connector
 */


if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

/**
 * a result array when funcitoning as SOAP interface
 */
$results = array();

/**
 * include connector config file
 */
require('modules/ZMerge/inc.config.php');

/**
 *  include some definitions
 */
require('modules/ZMerge/inc.defs.php');
require('modules/ZMerge/inc.utils.php');


/**
 * load sugar enviroment
 */
global $sugar_version, $timedate, $db, $locale, $current_user, $HTTP_RAW_POST_DATA, $results, $server, $NAMESPACE;


echoString("<h2>Z-Merge SugarCRM - Connector check</h2>");
echoString("<h4>basic configuration</h4>". SG_IGNORE_VERSION);

s("SugarCRM environment loaded");
if ($sugar_version) s("", true); else s("", false);

s("load configuration");
if (SG_WAIT_FOR_EVENTS) s("", true); else s("", false);


s("loading libraries");
include_once ('include/nusoap.php');
require_once ('modules/Tasks/Task.php');
require_once ('modules/Contacts/Contact.php');
require_once ('modules/Users/User.php');
require_once ('modules/Calls/Call.php');
require_once ('modules/Meetings/Meeting.php');
require_once ('modules/Notes/Note.php');
require('inc.soap.php');
require('inc.functions.php');
s("", (function_exists("getUTC") && function_exists("getChanges") ));


s("Check SugarCRM version");
// check if the installed sugar version is supported
if (! isCompatibleSugarVersion($sugar_version)) {
    $errString = "<font color=red>This version of the Z-Merge SugarCRM connector was not designed and tested for SugarCRM ". $sugar_version . "</font>";

    if (! isSOAP()) {
        echoString($errString);
        exit(0);
    }
    else {
        s($errString, false);
        asSOAP();
        exit(0);
    }
}
s($sugar_version, true);


s("check database");
s("", checkDatabase());


s("logged user is a SugarCRM admin");
if (isAdmin())  s("", true);
else            s("The complete functionality will only be available with admin credentials!", false);


echoString("<br><h4>connector status</h4>");
s("perform ping");
$ping = ping();
s("", ($ping['alive'] == "true"));


echoString("<br><h4>Z-Merge API</h4>");
s("select a change");
$row = getLastModId();
if ($row === false) {
    s("Please create/modify a 'Note', 'Contact', 'Call', 'Meeting' or a 'Task' and re-check.", false);

    if (isSOAP()) asSOAP();
    die();
}
else {
    s("<i>modid: {$row['id']}</i>", true);
    // add modid to the SOAP return array
    $results[] = array('name' => 'modid',   'value' => $row['id']);

}


s("get change using Z-Merge API");
if (isAdmin()) {
    $obj = getChanges ("nosession", ($row['id']-1) , true);
    s("", (is_array($obj) && is_array($obj[0])));
}
else s("skipped, no admin user", false);

s("writing object using Z-Merge API");

if (isAdmin()) {
    $GLOBALS['zmerge_change'] = 1;

    // correct data structure
    $nvarray = $obj[0]['data'][0]['name_value_array'];
    $obj[0]['data'][0] = $nvarray;
    // assign it to the same user
    if (is_array($nvarray)) foreach($nvarray as $a) if ($a['name'] == 'assigned_user_name' || $a['name'] == 'sugar_user_name') $obj[0]['userid'] = $a['value'];
}

if (isset($obj[0]['userid'])) {
    $re = setChanges ("nosession", $obj);
    s("{$re[0]['message']}", ($re[0]['status'] == 'update_ok' ));
}
else
    s("skipped", false);



s("Z-Merge loop detection");
if (isAdmin()) {
    $row2 = getLastModId($row['crmid']);
    s("" , ($row['id'] < $row2['id'] && $row2['zmerge'] == 1));
}
else s("skipped", false);

// showing some stats about the system
$contacts = getNrOfObjects("contacts");
$notes = getNrOfObjects("notes");
$tasks = getNrOfObjects("tasks");
$meetings = getNrOfObjects("meetings");
$calls = getNrOfObjects("calls");

echoString("Existing objects in the system:<br>");
echoString("&nbsp;&nbsp;&nbsp;&nbsp;Contacts: ". $contacts. "<br>");
echoString("&nbsp;&nbsp;&nbsp;&nbsp;Notes: ". $notes. "<br>");
echoString("&nbsp;&nbsp;&nbsp;&nbsp;Tasks: ". $tasks. "<br>");
echoString("&nbsp;&nbsp;&nbsp;&nbsp;Appointments: ". ($meetings+$calls) . " ==> Calls: $calls, Meetings: $meetings". "<br>");

// add the values to the SOAP return array
$results[] = array('name' => 'contacts',    'value' => $contacts);
$results[] = array('name' => 'notes',       'value' => $notes);
$results[] = array('name' => 'tasks',       'value' => $tasks);
$results[] = array('name' => 'appointments','value' => ($meetings+$calls));



// add userlist to returning array
addUserList();


echoString("<br><h4>Test connector completed</h4></div>");

/**
 * Build SOAP environment if called that way
 */
if (isSOAP()) asSOAP();



/**
 * Some helper functions
 */



/**
 * returns the check result as SOAP array
 *
 * @return complex-array()
 */
function check() {
    global $results;
    return $results;
}

/**
 * executes that script as a SOAP connector returning XML
 */
function asSOAP() {
    global $HTTP_RAW_POST_DATA, $results, $server, $NAMESPACE;

    $server->register(    'check',
                        array (),                                            // input parameters
                        array('name_value_array'=>'tns:name_value_list'),    // output parameters
                        $NAMESPACE,                                          // namespace
                        $NAMESPACE.'#check',                                 // soapaction
                        'rpc',                                               // style
                        'encoded',                                           // use
                        'Returns the status of the installation as xml-soap' // documentation
    );

    /**
     * Begin the HTTP listener service and exit.
     */
    $server->service($HTTP_RAW_POST_DATA);
    exit(0);
}

/**
 * returns the last event of the logged events
 *
 * @param $crmid    if set, only the last event for that object will be returned
 * @return resultSet
 */
function getLastModId($crmid = false) {
    global $db;

    $moreSQL = "";
    if ($crmid) $moreSQL = "AND crmid = '$crmid'";

    $stmt = "SELECT crmid, id, modified, type, zmerge " .
        "FROM zmerge WHERE id > 0 AND type IN ('meetings','tasks','contacts','notes', 'calls') $moreSQL ORDER BY `id` DESC LIMIT 0,1";

    $result = $db->query($stmt);
    if (!empty($result)) {
        $modid =  $db->fetchByAssoc($result);
        if (is_numeric($modid['id']))
            return $modid;
        else
            return array('id'=> 0);
    }
    else
        return false;
}

/**
 * returns number of a certain type of object in the system
 *
 * @param $tablename    in SugarCRM the objectname is the tablename
 * @return int
 */
function getNrOfObjects($tablename) {
    global $db;

    if (!$tablename || !
        ($tablename == "contacts" ||
         $tablename == "notes" ||
         $tablename == "tasks" ||
         $tablename == "meetings" ||
         $tablename == "calls"))
                return 0;

    $stmt = "SELECT COUNT(*) as nr FROM $tablename WHERE deleted = 0";
    $result = $db->query($stmt);

    if (!empty($result)) {
        $r = $db->fetchByAssoc($result);
        return $r['nr'];
    }
    return 0;
}

/**
 * adds all users of the system to the returning soap array
 *
 * @return void
 */
function addUserList() {
    global $db, $results, $current_user;

    $stmt = "SELECT user_name FROM users WHERE deleted = 0";
    $result = $db->query($stmt);

    if (!empty($result)) {
        $c = 0;
        while($r = $db->fetchByAssoc($result)) {
            // do not display sugar users to a non admin user
            if (!isAdmin() && $r['user_name'] != $current_user->user_name) continue;

            $results[] = array('name' => 'usr_'. $c, 'value' => $r['user_name']);
            $c++;
        }
    }
}

/**
 * prints status text to the STDOUT
 *
 * @param $string    string with additional informations
 * @param $flag        boolean informing the status of the operation
 * @return resultSet
 */
$testFlag = true;
$lastString = "";
function s($string, $flag = false) {
    global $testFlag, $lastString, $results;

    if ($testFlag && !$flag)  {
        if (isSOAP()) {
            $results[] = array('name' => $lastString,       'value' => "failed");
            if ($string)
                $results[] = array('name' => $lastString . "2", 'value' => $string);
        }
        else echo "<font color=red>failed <b>$string</b></font><br>";
        $lastString = "";
        $testFlag = false;
    }
    else if  ($testFlag) {
        if (isSOAP()) $results[] = array('name' => $lastString, 'value' => (($string)?$string. " - ":"") . "ok");
        else echo $string . " ok <br>";
        $testFlag = false;
    }
    else {
        if ($string) {
            echoString($string . ": ");
            $lastString = $string;
            $testFlag = true;
        }
    }
}


/**
 * prints a text to STDOUT
 *
 * @param $string    string with additional informations
 * @return void
 */
function echoString($string) {
    if (!isSOAP()) echo $string;
}


/**
 * informs if the script is executed as SOAP interface or as ordinary script
 *
 * @return boolean
 */
function isSOAP() {
    global $_GET;
    return isset($_GET['soap']);
}


/**
 * Checks if the SugarCRM user is an admin user
 *
 * @return boolean
 */
function isAdmin() {
    global $current_user;
    return ($current_user instanceOf User && $current_user->is_admin);
}

?>